@@ -2,16 +2,23 @@ package RDF::iCalendar::Exporter;
 
 use 5.008;
 use base qw[RDF::vCard::Exporter];
-use common::sense;
+use strict;
+use warnings;
+no warnings qw(uninitialized);
 
 use DateTime;
 use MIME::Base64 qw[];
 use RDF::iCalendar::Entity;
 use RDF::iCalendar::Line;
-use RDF::TrineShortcuts qw[:all];
+use RDF::TrineX::Functions
+	-shortcuts,
+	statement => { -as => 'rdf_statement' },
+	iri       => { -as => 'rdf_resource' };
 use Scalar::Util qw[blessed];
 use URI;
 
+require RDF::vCard;
+
 # kinda constants
 sub I    { return 'http://www.w3.org/2002/12/cal/icaltzd#' . shift; }
 sub IX   { return 'http://buzzword.org.uk/rdf/icaltzdx#' . shift; }
@@ -20,9 +27,16 @@ sub V    { return 'http://www.w3.org/2006/vcard/ns#' . shift; }
 sub VX   { return 'http://buzzword.org.uk/rdf/vcardx#' . shift; }
 sub XSD  { return 'http://www.w3.org/2001/XMLSchema#' . shift; }
 
+sub flatten_node
+{
+	my $node = shift;
+	return $node->value if $node->is_resource || $node->is_literal;
+	return $node->as_ntriples;
+}
+
 use namespace::clean;
 
-our $VERSION = '0.000_00';
+our $VERSION = '0.005';
 our $PRODID  = sprintf("+//IDN cpan.org//NONSGML %s v %s//EN", __PACKAGE__, $VERSION);
 
 our %cal_dispatch = (
@@ -48,9 +62,9 @@ our %dispatch = (
 	I('location')    => \&_prop_export_location,
 	I('rrule')       => \&_prop_export_Recur,
 	I('exrule')      => \&_prop_export_Recur,
+	I('valarm')      => \&_prop_export_valarm,
+	I('freebusy')    => \&_prop_export_freebusy,
 	# RELATED-TO
-	# VALARM
-	# FREEBUSY
 	);
 
 our %list_dispatch = (
@@ -62,7 +76,7 @@ our %list_dispatch = (
 	IX('category')   => ['categories', \&_value_export_category],
 	);
 	
-sub rebless
+sub _rebless
 {
 	my ($self, $thing) = @_;
 	if ($thing->isa('RDF::vCard::Line'))
@@ -75,7 +89,7 @@ sub rebless
 	}
 }
 
-sub debug
+sub _debug
 {
 #	my ($self, @debug) = @_;
 #	printf(@debug);
@@ -142,7 +156,11 @@ sub export_calendar
 		and    ref($cal_dispatch{$triple->predicate->uri}) eq 'CODE')
 		{
 			my $code = $cal_dispatch{$triple->predicate->uri};
-			$ical->add($code->($self, $model, $triple));
+			my $r    = $code->($self, $model, $triple);
+			if (blessed($r) and $r->isa('RDF::iCalendar::Line'))
+				{ $ical->add($r); }
+			elsif (blessed($r) and $r->isa('RDF::iCalendar::Entity'))
+				{ $ical->add_component($r); }
 		}
 		elsif ((substr($triple->predicate->uri, 0, length(&I)) eq &I
 		or substr($triple->predicate->uri, 0, length(&IX)) eq &IX))
@@ -199,39 +217,43 @@ sub export_component
 	
 	my $c = RDF::iCalendar::Entity->new( profile=>$profile );
 	
-	$self->debug("COMPONENT: %s", flatten_node($subject));
+	$self->_debug("COMPONENT: %s", flatten_node($subject));
 	
 	my $lists = {};
 	
 	my $triples = $model->get_statements($subject, undef, undef);
 	while (my $triple = $triples->next)
 	{
-		$self->debug("  %s %s", $triple->predicate->sse, $triple->object->sse);
+		$self->_debug("  %s %s", $triple->predicate->sse, $triple->object->sse);
 	
 		if (defined $dispatch{$triple->predicate->uri}
 		and ref($dispatch{$triple->predicate->uri}) eq 'CODE')
 		{
-			$self->debug("   -> dispatch");
+			$self->_debug("   -> dispatch");
 			my $code = $dispatch{$triple->predicate->uri};
-			$c->add($code->($self, $model, $triple));
+			my $r    = $code->($self, $model, $triple);
+			if (blessed($r) and $r->isa('RDF::iCalendar::Line'))
+				{ $c->add($r); }
+			elsif (blessed($r) and $r->isa('RDF::iCalendar::Entity'))
+				{ $c->add_component($r); }
 		}
 		elsif (defined $list_dispatch{$triple->predicate->uri}
 		and ref($list_dispatch{$triple->predicate->uri}) eq 'ARRAY')
 		{
-			$self->debug("   -> list_dispatch");
+			$self->_debug("   -> list_dispatch");
 			my ($listname, $code) = @{ $list_dispatch{$triple->predicate->uri} };
 			push @{ $lists->{$listname} }, $code->($self, $model, $triple);
 		}
 		elsif ((substr($triple->predicate->uri, 0, length(&I)) eq &I
 		or substr($triple->predicate->uri, 0, length(&IX)) eq &IX))
 		{
-			$self->debug("   -> default");
+			$self->_debug("   -> default");
 			$c->add($self->_prop_export_simple($model, $triple))
 				unless $triple->object->is_blank;
 		}
 		else
 		{
-			$self->debug("   -> NO ACTION");
+			$self->_debug("   -> NO ACTION");
 		}
 	}
 	
@@ -246,11 +268,21 @@ sub export_component
 	return $c;
 }
 
+sub _prop_export_valarm
+{
+	my ($self, $model, $triple) = @_;
+	
+	unless ($triple->object->is_literal)
+	{
+		return $self->export_component($model, $triple->object);
+	}
+}
+
 sub _prop_export_simple
 {
 	my ($self, $model, $triple) = @_;
 	my $rv = $self->SUPER::_prop_export_simple($model, $triple);
-	return $self->rebless($rv);
+	return $self->_rebless($rv);
 }
 
 # iCalendar forces different datetime/date formats than
@@ -440,11 +472,11 @@ sub _prop_export_location
 {
 	my ($self, $model, $triple) = @_;
 
-	$self->debug("      Location: %s", flatten_node($triple->object));
+	$self->_debug("      Location: %s", flatten_node($triple->object));
 
 	if ($triple->object->is_literal)
 	{
-		$self->debug("       -> literal");
+		$self->_debug("       -> literal");
 		return $self->_prop_export_simple($model, $triple);
 	}
 
@@ -460,7 +492,7 @@ sub _prop_export_location
 			)
 		)
 	{
-		$self->debug("       -> vcard");
+		$self->_debug("       -> vcard");
 		my $card = $self->export_card($model, $triple->object);
 		return RDF::iCalendar::Line->new(
 			property => 'location',
@@ -488,8 +520,8 @@ sub _prop_export_location
 			)
 		)
 	{
-		$self->debug("       -> adr");
-		my $line = $self->rebless( $self->_prop_export_adr($model, $triple) );
+		$self->_debug("       -> adr");
+		my $line = $self->_rebless( $self->_prop_export_adr($model, $triple) );
 		$line->{property} = 'location';
 		return $line;
 	}
@@ -506,8 +538,8 @@ sub _prop_export_location
 			)
 		)
 	{
-		$self->debug("       -> geo");
-		my $line = $self->rebless( $self->SUPER::_prop_export_geo($model, $triple) );
+		$self->_debug("       -> geo");
+		my $line = $self->_rebless( $self->SUPER::_prop_export_geo($model, $triple) );
 		$line->{property} = 'location';
 		return $line;
 	}
@@ -780,6 +812,46 @@ sub _prop_export_Recur
 		);
 }
 
+sub _prop_export_freebusy
+{
+	my ($self, $model, $triple) = @_;
+
+	if ($triple->object->is_literal)
+	{
+		return RDF::iCalendar::Line->new(
+			property => 'freebusy',
+			value    => $triple->object->literal_value,
+			type_parameters => { fbtype => 'BUSY' },
+			);
+	}
+
+	my @values = sort map
+		{ $_->literal_value }
+		grep
+		{ $_->is_literal }
+		$model->objects_for_predicate_list(
+			$triple->object,
+			rdf_resource(RDF('value')),
+			);
+	
+	my ($fbtype) = map
+		{ uc $_->literal_value }
+		grep
+		{ $_->is_literal }
+		$model->objects_for_predicate_list(
+			$triple->object,
+			rdf_resource(I('fbtype')),
+			rdf_resource(IX('fbtype')),
+			);
+
+	return RDF::iCalendar::Line->new(
+		property => 'freebusy',
+		value    => [[ @values ]],
+		type_parameters => { fbtype => $fbtype || 'BUSY' },
+		);
+
+}
+
 
 1;
 
@@ -893,8 +965,13 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright 2011 Toby Inkster
+Copyright 2011, 2013 Toby Inkster
 
-This library is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=head1 DISCLAIMER OF WARRANTIES
+
+THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
+WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
+MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
